package de.ozgcloud.eingang.fim;

import de.ozgcloud.common.test.ITCase;
import de.ozgcloud.eingang.common.formdata.FormData;
import de.ozgcloud.eingang.common.formdata.IncomingFile;
import de.ozgcloud.eingang.common.formdata.IncomingFileGroup;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.autoconfigure.ImportAutoConfiguration;

import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.test.context.ActiveProfiles;

import org.w3c.dom.Document;
import org.xml.sax.SAXException;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;
import java.io.File;
import java.io.IOException;
import java.util.Map;
import org.junit.jupiter.api.Test;

import static org.assertj.core.api.Assertions.assertThat;
import static org.assertj.core.api.AssertionsForClassTypes.assertThatThrownBy;

@ITCase
@ActiveProfiles({ "itcase", "test" })
@ImportAutoConfiguration
@SpringBootTest(classes= FimTestConfig.class)
public class FimServiceITCase {

    @Autowired
    private FimService fimService;

    @Autowired
    private FimBasedMapper fimBasedMapper;

    @Test
    void shouldFailOnEmptyScheme() {
        assertThatThrownBy(() -> {
            final Document document = loadDocument("src/test/resources/test1.xml");
            fimService.transformDocument(document, FormData.builder().build());
        }).isInstanceOf(FimException.class).hasMessage("XML Document does not provide a scheme");
    }

    @Test
    void shouldNoFindInvalidScheme() {
        final FimScheme scheme = fimService.getSchemeForIdentifier("test");

        assertThat(FimSchemeIdentifier.fromString("unknown")).isEqualTo(scheme.getIdentifier());
    }

    @Test
    void shouldFindVersammlungsScheme() {
        final FimScheme scheme = fimService.getSchemeForIdentifier("urn:xoev-de:xfall:standard:fim-s17000652_1.4");
        assertThat(scheme).isNotNull();
    }

    @Test
    void shouldFindTest2Scheme() {
        final FimScheme scheme = fimService.getSchemeForIdentifier("test2");
        assertThat(scheme).isNotNull();
    }

    @Test
    void shouldTransformSimpleDocument() throws ParserConfigurationException, IOException, SAXException {
        final Document document = loadDocument("src/test/resources/test2.xml");
        final FormData formData = fimService.transformDocument(document, FormData.builder().build());

        final Map<String, Object> expected = Map.of(
                "xs:fim.S1235", Map.of("label", "Testkey", "value", "Testvalue"),
                "xs:fim.S1236", Map.of("label", "xs:fim.S1236", "value", "Testvalue 2")
        );
        assertThat(expected).isEqualTo(formData.getFormData());
    }

    @Test
    void shouldTransformDocument() throws ParserConfigurationException, IOException, SAXException {
        final Document document = loadDocument("src/test/resources/S17000652V1.4_test01.xml");
        FormData formData = fimService.transformDocument(document, FormData.builder().build());

        assertThat(formData).isNotNull();

        final Map<String, Object> expected = Map.of(
                "xfd:G17003529", Map.of(
                        "label","EfA|SH Standard",
                        "value", Map.of(
                                "xfd:G05001479", Map.of(
                                        "label","nachrichtenkopf",
                                        "value", Map.of(
                                                "xfd:G05001480", Map.of(
                                                        "label","identifikation.nachricht",
                                                        "value", Map.of(
                                                                "xfd:F05002750", Map.of("label", "nachrichtenUUID", "value", "d447e43a-5723-4821-a170-cb44d2dbf143"),
                                                                "xfd:F05002751", Map.of("label", "erstellungszeitpunkt", "value", "2022-08-15T09:30:47"),
                                                                "xfd:F05002752", Map.of("label", "nachrichtentyp", "value", "fim.S17000652.17000652001004"),
                                                                "xfd:F05002753", Map.of("label", "dienstname", "value", "urn:fim:Versammlungsanzeige:1.4")
                                                        )
                                                ),
                                                "xfd:G05001481", Map.of(
                                                        "label","Leser",
                                                        "value", Map.of(
                                                                "xfd:F05002754", Map.of("label", "Organisationsname", "value", "Celle"),
                                                                "xfd:F05002755", Map.of("label", "Organisationsschlüssel", "value", "vbe:010550120100"),
                                                                "xfd:F05002756", Map.of("label", "Kategorie", "value", "Versammlungsbehörde")
                                                        )
                                                ),
                                                "xfd:G05001482", Map.of(
                                                        "label","Autor",
                                                        "value", Map.of(
                                                                "xfd:F05002754", Map.of("label", "Organisationsname", "value", "OSI-Onlinedienst Niedersachsen Versammlungsanzeige"),
                                                                "xfd:F05002755", Map.of("label", "Organisationsschlüssel", "value", "vbe:010550120100"),
                                                                "xfd:F05002756", Map.of("label", "Kategorie", "value", "Engagement- und Hobbyportal")
                                                        )
                                                )
                                        )
                                ),
                                "xfd:F17005454", Map.of("label", "Datenschutzhinweis DSGVO", "value", "true"),
                                "xfd:F17005455", Map.of("label", "Zustimmung zu einem digitalen Bescheid", "value", "true"),
                                "xfd:F17005533", Map.of("label", "UUID", "value", "String")
                        )
                ),
                "xfd:F17009191", Map.of("label", "Anzeige durch Person", "value", "true"),
                "xfd:F17003371", Map.of("label", "Anzeigenart", "value", "String")
        );

        assertThat(expected).isEqualTo(formData.getFormData());
    }

    @Test
    void shouldFallbackUnknownScheme() {
        final FormData initialFormData = FormData.builder().attachment(
                IncomingFileGroup.builder()
                        .file(IncomingFile.builder().name("src/test/resources/test3/Antrag.xml").file(new File("src/test/resources/test3/Antrag.xml")).build())
                        .file(IncomingFile.builder().name("src/test/resources/test3/fim_xtaMetadata.xml").build()).build()).build();

        FormData formData = fimBasedMapper.parseFormData(initialFormData);

        assertThat(formData).isNotNull();

        final Map<String, Object> expected = Map.of(
                "xfd:G17003529", Map.of(
                        "label","xfd:G17003529",
                        "value", Map.of(
                                "xfd:G05001479", Map.of(
                                        "label","xfd:G05001479",
                                        "value", Map.of(
                                                "xfd:G05001480", Map.of(
                                                        "label","xfd:G05001480",
                                                        "value", Map.of(
                                                                "xfd:F05002750", Map.of("label", "xfd:F05002750", "value", "d447e43a-5723-4821-a170-cb44d2dbf143"),
                                                                "xfd:F05002751", Map.of("label", "xfd:F05002751", "value", "2022-08-15T09:30:47"),
                                                                "xfd:F05002752", Map.of("label", "xfd:F05002752", "value", "fim.S17000652.17000652001004"),
                                                                "xfd:F05002753", Map.of("label", "xfd:F05002753", "value", "urn:fim:Versammlungsanzeige:1.4")
                                                        )
                                                ),
                                                "xfd:G05001481", Map.of(
                                                        "label","xfd:G05001481",
                                                        "value", Map.of(
                                                                "xfd:F05002754", Map.of("label", "xfd:F05002754", "value", "Celle"),
                                                                "xfd:F05002755", Map.of("label", "xfd:F05002755", "value", "vbe:010550120100"),
                                                                "xfd:F05002756", Map.of("label", "xfd:F05002756", "value", "Versammlungsbehörde")
                                                        )
                                                ),
                                                "xfd:G05001482", Map.of(
                                                        "label","xfd:G05001482",
                                                        "value", Map.of(
                                                                "xfd:F05002754", Map.of("label", "xfd:F05002754", "value", "OSI-Onlinedienst Niedersachsen Versammlungsanzeige"),
                                                                "xfd:F05002755", Map.of("label", "xfd:F05002755", "value", "vbe:010550120100"),
                                                                "xfd:F05002756", Map.of("label", "xfd:F05002756", "value", "Engagement- und Hobbyportal")
                                                        )
                                                )
                                        )
                                ),
                                "xfd:F17005454", Map.of("label", "xfd:F17005454", "value", "true"),
                                "xfd:F17005455", Map.of("label", "xfd:F17005455", "value", "true"),
                                "xfd:F17005533", Map.of("label", "xfd:F17005533", "value", "String")
                        )
                ),
                "xfd:F17009191", Map.of("label", "xfd:F17009191", "value", "true"),
                "xfd:F17003371", Map.of("label", "xfd:F17003371", "value", "String")
        );

        assertThat(expected).isEqualTo(formData.getFormData());
    }

    private Document loadDocument(final String path) throws ParserConfigurationException, IOException, SAXException {
        final DocumentBuilder builder = DocumentBuilderFactory.newInstance().newDocumentBuilder();
        return builder.parse(new File(path));
    }
}
