package de.ozgcloud.eingang.xta;

import static org.assertj.core.api.Assertions.*;
import static org.mockito.Mockito.*;

import java.util.stream.Stream;

import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.mockito.Mock;

class XtaMessageMetadataRemoteIteratorTest {

	private XtaMessageMetaDatasAndHeader messageMetaDatasAndHeader = XtaMessageMetaDatasAndHeaderTestFactory.create();

	@Mock
	private XtaRemoteService remoteService;

	@BeforeEach
	void setup() {
		when(remoteService.getMessagesMetadata()).thenReturn(messageMetaDatasAndHeader);
	}

	@Nested
	class TestIteratorInitialization {

		@Test
		void shouldCallGetMessagesMetadata() {
			new XtaMessageMetadataRemoteIterator(remoteService);

			verify(remoteService).getMessagesMetadata();
		}

		@Test
		void shouldCallRemoteMessageIterator() {
			when(remoteService.getMessagesMetadata()).thenReturn(messageMetaDatasAndHeader);

			Object remoteIterator = spy(new XtaMessageMetadataRemoteIterator(remoteService));

			assertThat(remoteIterator).extracting("remoteMessageIterator").isNotNull();
		}
	}

	@Nested
	class TestHasNext {

		@Test
		void shouldReturnTrueAfterInitialization() {
			var remoteIterator = new XtaMessageMetadataRemoteIterator(remoteService);

			assertThat(remoteIterator.hasNext()).isTrue();
		}

		@Test
		void shouldReturnTrueWhenMoreMessagesAvailable() {
			var messageMetadataAndHeader = XtaMessageMetaDatasAndHeaderTestFactory.createBuilder().moreMessagesAvailable(true).build();
			when(remoteService.getMessagesMetadata()).thenReturn(messageMetadataAndHeader);

			var remoteIterator = new XtaMessageMetadataRemoteIterator(remoteService);

			assertThat(remoteIterator.hasNext()).isTrue();
		}

		@Test
		void shouldCallLoadNextMessages() {
			initTest();
			var remoteIterator = spy(new XtaMessageMetadataRemoteIterator(remoteService));

			remoteIterator.hasNext();

			verify(remoteIterator).loadNextMessages();
		}

		private void initTest() {
			var messageMetadataAndHeader = XtaMessageMetaDatasAndHeaderTestFactory.createBuilder()
					.messages(Stream.empty()).moreMessagesAvailable(true).build();
			when(remoteService.getMessagesMetadata()).thenReturn(messageMetadataAndHeader);
			var nextMessageMetadataAndHeader = XtaMessageMetaDatasAndHeaderTestFactory.createBuilder().msgBoxRequestID("id").build();
			when(remoteService.getNextMessagesMetadata(any())).thenReturn(nextMessageMetadataAndHeader);
		}

		@Test
		void shouldReturnFalseWhenNoMoreMessagesAvailable() {
			var messageMetadataAndHeader = XtaMessageMetaDatasAndHeaderTestFactory.createBuilder().messages(Stream.empty()).build();
			when(remoteService.getMessagesMetadata()).thenReturn(messageMetadataAndHeader);

			var remoteIterator = new XtaMessageMetadataRemoteIterator(remoteService);

			assertThat(remoteIterator.hasNext()).isFalse();
		}
	}

	@Nested
	class TestLoadNextMessages {

		private XtaMessageMetaDatasAndHeader nextMessageMetadataAndHeader = XtaMessageMetaDatasAndHeaderTestFactory.createBuilder().msgBoxRequestID("id").build();

		@BeforeEach
		void setup() {
			when(remoteService.getNextMessagesMetadata(any())).thenReturn(nextMessageMetadataAndHeader);
		}

		@Test
		void shouldCallGetMessages() {
			new XtaMessageMetadataRemoteIterator(remoteService).loadNextMessages();

			verify(remoteService).getNextMessagesMetadata(messageMetaDatasAndHeader.getMsgBoxRequestID());
		}

		@Test
		void shouldCallGetRemoteMessageIterator() {
			var remoteIterator = spy(new XtaMessageMetadataRemoteIterator(remoteService));

			remoteIterator.loadNextMessages();

			verify(remoteIterator).getRemoteMessageIterator(nextMessageMetadataAndHeader);
		}
	}
}